#include "atlas_misc.h"
#ifndef PFYDIST
   #define PFYDIST 12
#endif
#ifndef PFXDIST
   #define PFXDIST 256
#endif
#ifndef PFADIST
   #define PFADIST 256
#endif

#if !defined(ATL_3DNow) && !defined(ATL_SSE1) && \
    (PFADIST != 0 || PFXDIST != 0 || PFYDIST != 0)
   #include "atlas_prefetch.h"
#endif

#if defined(ATL_3DNow) || defined(ATL_SSE1)
   #ifndef PFIY
      #define PFIY prefetchnta
   #endif
   #ifndef PFIX
      #define PFIX prefetchnta
   #endif
   #ifndef PFIA
      #ifdef ATL_3DNow
         #define PFIA prefetchw
      #else
         #define PFIA prefetcht0
      #endif
   #endif
#endif
/*
 * X & A are prefetched in M loop PF[A,X]DIST (in bytes) ahead
 */
#if PFADIST == 0                /* flag for no prefetch */
   #define prefA(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefA(mem) __asm__ __volatile__ \
         (Mstr(PFIA) " %0" : : "m" (*(((char *)(mem))+PFADIST)))
   #else
      #if PFLVL == 2
         #define prefA(mem) ATL_pfl2W(((char*)mem)+PFADIST)
      #else
         #define prefA(mem) ATL_pfl1W(((char*)mem)+PFADIST)
      #endif
   #endif
#endif
#if PFXDIST == 0                /* flag for no prefetch */
   #define prefX(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefX(mem) __asm__ __volatile__ \
         (Mstr(PFIX) " %0" : : "m" (*(((char *)(mem))+PFXDIST)))
   #else
      #if PFLVL == 2
         #define prefX(mem) ATL_pfl2R(((char*)mem)+PFXDIST)
      #else
         #define prefX(mem) ATL_pfl1R(((char*)mem)+PFXDIST)
      #endif
   #endif
#endif
/*
 * Y is prefetched in N-loop, and always fetches next NU piece
 */
#if PFYDIST == 0                /* flag for no prefetch */
   #define prefY(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefY(mem) \
         __asm__ __volatile__ (Mstr(PFIY) " %0" : : "m" (*(((char *)(mem)))))
   #else
      #if PFLVL == 2
         #define prefY(mem) ATL_pfl2R(mem)
      #else
         #define prefY(mem) ATL_pfl1R(mem)
      #endif
   #endif
#endif
#ifndef ATL_CINT
   #define ATL_CINT const int
#endif
#ifndef ATL_INT
   #define ATL_INT int
#endif
void ATL_UGEMV(ATL_CINT M0, ATL_CINT N, const TYPE *A, ATL_CINT lda0,
               const TYPE *X, TYPE *Y)
/*
 * 24x12 unrolled mvt_c.
 * Extracted from ATLAS/tune/blas/gemv/atlas-l2g.base
 */
{
   ATL_CINT N12=(N/12)*12, M=M0+M0, M24=((M0/24)*24)<<1,
            lda=lda0+lda0, lda12=lda*12;
   ATL_INT j;

   for (j=N12; j; j -= 12, A += lda12, Y += 24)
   {
      const float *A0=A, *A1=A0+lda, *A2=A1+lda, *A3=A2+lda, *A4=A3+lda,
                  *A5=A4+lda, *A6=A5+lda, *A7=A6+lda, *A8=A7+lda, *A9=A8+lda,
                  *A10=A9+lda, *A11=A10+lda;
      register float ry0=ATL_rzero, iy0=ATL_rzero, ry1=ATL_rzero, iy1=ATL_rzero,
                     ry2=ATL_rzero, iy2=ATL_rzero, ry3=ATL_rzero, iy3=ATL_rzero,
                     ry4=ATL_rzero, iy4=ATL_rzero, ry5=ATL_rzero, iy5=ATL_rzero,
                     ry6=ATL_rzero, iy6=ATL_rzero, ry7=ATL_rzero, iy7=ATL_rzero,
                     ry8=ATL_rzero, iy8=ATL_rzero, ry9=ATL_rzero, iy9=ATL_rzero,
                     ry10=ATL_rzero, iy10=ATL_rzero, ry11=ATL_rzero,
                     iy11=ATL_rzero;
      ATL_INT i;
      prefY(Y+24+24-1);
      for (i=0; i < M24; i += 48)
      {
         const register float ra0_0=A0[i+0], ia0_0=A0[i+1], ra1_0=A0[i+2],
                              ia1_0=A0[i+3], ra2_0=A0[i+4], ia2_0=A0[i+5],
                              ra3_0=A0[i+6], ia3_0=A0[i+7], ra4_0=A0[i+8],
                              ia4_0=A0[i+9], ra5_0=A0[i+10], ia5_0=A0[i+11],
                              ra6_0=A0[i+12], ia6_0=A0[i+13], ra7_0=A0[i+14],
                              ia7_0=A0[i+15], ra8_0=A0[i+16], ia8_0=A0[i+17],
                              ra9_0=A0[i+18], ia9_0=A0[i+19], ra10_0=A0[i+20],
                              ia10_0=A0[i+21], ra11_0=A0[i+22], ia11_0=A0[i+23],
                              ra12_0=A0[i+24], ia12_0=A0[i+25], ra13_0=A0[i+26],
                              ia13_0=A0[i+27], ra14_0=A0[i+28], ia14_0=A0[i+29],
                              ra15_0=A0[i+30], ia15_0=A0[i+31], ra16_0=A0[i+32],
                              ia16_0=A0[i+33], ra17_0=A0[i+34], ia17_0=A0[i+35],
                              ra18_0=A0[i+36], ia18_0=A0[i+37], ra19_0=A0[i+38],
                              ia19_0=A0[i+39], ra20_0=A0[i+40], ia20_0=A0[i+41],
                              ra21_0=A0[i+42], ia21_0=A0[i+43], ra22_0=A0[i+44],
                              ia22_0=A0[i+45], ra23_0=A0[i+46], ia23_0=A0[i+47],
                              ra0_1=A1[i+0], ia0_1=A1[i+1], ra1_1=A1[i+2],
                              ia1_1=A1[i+3], ra2_1=A1[i+4], ia2_1=A1[i+5],
                              ra3_1=A1[i+6], ia3_1=A1[i+7], ra4_1=A1[i+8],
                              ia4_1=A1[i+9], ra5_1=A1[i+10], ia5_1=A1[i+11],
                              ra6_1=A1[i+12], ia6_1=A1[i+13], ra7_1=A1[i+14],
                              ia7_1=A1[i+15], ra8_1=A1[i+16], ia8_1=A1[i+17],
                              ra9_1=A1[i+18], ia9_1=A1[i+19], ra10_1=A1[i+20],
                              ia10_1=A1[i+21], ra11_1=A1[i+22], ia11_1=A1[i+23],
                              ra12_1=A1[i+24], ia12_1=A1[i+25], ra13_1=A1[i+26],
                              ia13_1=A1[i+27], ra14_1=A1[i+28], ia14_1=A1[i+29],
                              ra15_1=A1[i+30], ia15_1=A1[i+31], ra16_1=A1[i+32],
                              ia16_1=A1[i+33], ra17_1=A1[i+34], ia17_1=A1[i+35],
                              ra18_1=A1[i+36], ia18_1=A1[i+37], ra19_1=A1[i+38],
                              ia19_1=A1[i+39], ra20_1=A1[i+40], ia20_1=A1[i+41],
                              ra21_1=A1[i+42], ia21_1=A1[i+43], ra22_1=A1[i+44],
                              ia22_1=A1[i+45], ra23_1=A1[i+46], ia23_1=A1[i+47],
                              ra0_2=A2[i+0], ia0_2=A2[i+1], ra1_2=A2[i+2],
                              ia1_2=A2[i+3], ra2_2=A2[i+4], ia2_2=A2[i+5],
                              ra3_2=A2[i+6], ia3_2=A2[i+7], ra4_2=A2[i+8],
                              ia4_2=A2[i+9], ra5_2=A2[i+10], ia5_2=A2[i+11],
                              ra6_2=A2[i+12], ia6_2=A2[i+13], ra7_2=A2[i+14],
                              ia7_2=A2[i+15], ra8_2=A2[i+16], ia8_2=A2[i+17],
                              ra9_2=A2[i+18], ia9_2=A2[i+19], ra10_2=A2[i+20],
                              ia10_2=A2[i+21], ra11_2=A2[i+22], ia11_2=A2[i+23],
                              ra12_2=A2[i+24], ia12_2=A2[i+25], ra13_2=A2[i+26],
                              ia13_2=A2[i+27], ra14_2=A2[i+28], ia14_2=A2[i+29],
                              ra15_2=A2[i+30], ia15_2=A2[i+31], ra16_2=A2[i+32],
                              ia16_2=A2[i+33], ra17_2=A2[i+34], ia17_2=A2[i+35],
                              ra18_2=A2[i+36], ia18_2=A2[i+37], ra19_2=A2[i+38],
                              ia19_2=A2[i+39], ra20_2=A2[i+40], ia20_2=A2[i+41],
                              ra21_2=A2[i+42], ia21_2=A2[i+43], ra22_2=A2[i+44],
                              ia22_2=A2[i+45], ra23_2=A2[i+46], ia23_2=A2[i+47],
                              ra0_3=A3[i+0], ia0_3=A3[i+1], ra1_3=A3[i+2],
                              ia1_3=A3[i+3], ra2_3=A3[i+4], ia2_3=A3[i+5],
                              ra3_3=A3[i+6], ia3_3=A3[i+7], ra4_3=A3[i+8],
                              ia4_3=A3[i+9], ra5_3=A3[i+10], ia5_3=A3[i+11],
                              ra6_3=A3[i+12], ia6_3=A3[i+13], ra7_3=A3[i+14],
                              ia7_3=A3[i+15], ra8_3=A3[i+16], ia8_3=A3[i+17],
                              ra9_3=A3[i+18], ia9_3=A3[i+19], ra10_3=A3[i+20],
                              ia10_3=A3[i+21], ra11_3=A3[i+22], ia11_3=A3[i+23],
                              ra12_3=A3[i+24], ia12_3=A3[i+25], ra13_3=A3[i+26],
                              ia13_3=A3[i+27], ra14_3=A3[i+28], ia14_3=A3[i+29],
                              ra15_3=A3[i+30], ia15_3=A3[i+31], ra16_3=A3[i+32],
                              ia16_3=A3[i+33], ra17_3=A3[i+34], ia17_3=A3[i+35],
                              ra18_3=A3[i+36], ia18_3=A3[i+37], ra19_3=A3[i+38],
                              ia19_3=A3[i+39], ra20_3=A3[i+40], ia20_3=A3[i+41],
                              ra21_3=A3[i+42], ia21_3=A3[i+43], ra22_3=A3[i+44],
                              ia22_3=A3[i+45], ra23_3=A3[i+46], ia23_3=A3[i+47],
                              ra0_4=A4[i+0], ia0_4=A4[i+1], ra1_4=A4[i+2],
                              ia1_4=A4[i+3], ra2_4=A4[i+4], ia2_4=A4[i+5],
                              ra3_4=A4[i+6], ia3_4=A4[i+7], ra4_4=A4[i+8],
                              ia4_4=A4[i+9], ra5_4=A4[i+10], ia5_4=A4[i+11],
                              ra6_4=A4[i+12], ia6_4=A4[i+13], ra7_4=A4[i+14],
                              ia7_4=A4[i+15], ra8_4=A4[i+16], ia8_4=A4[i+17],
                              ra9_4=A4[i+18], ia9_4=A4[i+19], ra10_4=A4[i+20],
                              ia10_4=A4[i+21], ra11_4=A4[i+22], ia11_4=A4[i+23],
                              ra12_4=A4[i+24], ia12_4=A4[i+25], ra13_4=A4[i+26],
                              ia13_4=A4[i+27], ra14_4=A4[i+28], ia14_4=A4[i+29],
                              ra15_4=A4[i+30], ia15_4=A4[i+31], ra16_4=A4[i+32],
                              ia16_4=A4[i+33], ra17_4=A4[i+34], ia17_4=A4[i+35],
                              ra18_4=A4[i+36], ia18_4=A4[i+37], ra19_4=A4[i+38],
                              ia19_4=A4[i+39], ra20_4=A4[i+40], ia20_4=A4[i+41],
                              ra21_4=A4[i+42], ia21_4=A4[i+43], ra22_4=A4[i+44],
                              ia22_4=A4[i+45], ra23_4=A4[i+46], ia23_4=A4[i+47],
                              ra0_5=A5[i+0], ia0_5=A5[i+1], ra1_5=A5[i+2],
                              ia1_5=A5[i+3], ra2_5=A5[i+4], ia2_5=A5[i+5],
                              ra3_5=A5[i+6], ia3_5=A5[i+7], ra4_5=A5[i+8],
                              ia4_5=A5[i+9], ra5_5=A5[i+10], ia5_5=A5[i+11],
                              ra6_5=A5[i+12], ia6_5=A5[i+13], ra7_5=A5[i+14],
                              ia7_5=A5[i+15], ra8_5=A5[i+16], ia8_5=A5[i+17],
                              ra9_5=A5[i+18], ia9_5=A5[i+19], ra10_5=A5[i+20],
                              ia10_5=A5[i+21], ra11_5=A5[i+22], ia11_5=A5[i+23],
                              ra12_5=A5[i+24], ia12_5=A5[i+25], ra13_5=A5[i+26],
                              ia13_5=A5[i+27], ra14_5=A5[i+28], ia14_5=A5[i+29],
                              ra15_5=A5[i+30], ia15_5=A5[i+31], ra16_5=A5[i+32],
                              ia16_5=A5[i+33], ra17_5=A5[i+34], ia17_5=A5[i+35],
                              ra18_5=A5[i+36], ia18_5=A5[i+37], ra19_5=A5[i+38],
                              ia19_5=A5[i+39], ra20_5=A5[i+40], ia20_5=A5[i+41],
                              ra21_5=A5[i+42], ia21_5=A5[i+43], ra22_5=A5[i+44],
                              ia22_5=A5[i+45], ra23_5=A5[i+46], ia23_5=A5[i+47],
                              ra0_6=A6[i+0], ia0_6=A6[i+1], ra1_6=A6[i+2],
                              ia1_6=A6[i+3], ra2_6=A6[i+4], ia2_6=A6[i+5],
                              ra3_6=A6[i+6], ia3_6=A6[i+7], ra4_6=A6[i+8],
                              ia4_6=A6[i+9], ra5_6=A6[i+10], ia5_6=A6[i+11],
                              ra6_6=A6[i+12], ia6_6=A6[i+13], ra7_6=A6[i+14],
                              ia7_6=A6[i+15], ra8_6=A6[i+16], ia8_6=A6[i+17],
                              ra9_6=A6[i+18], ia9_6=A6[i+19], ra10_6=A6[i+20],
                              ia10_6=A6[i+21], ra11_6=A6[i+22], ia11_6=A6[i+23],
                              ra12_6=A6[i+24], ia12_6=A6[i+25], ra13_6=A6[i+26],
                              ia13_6=A6[i+27], ra14_6=A6[i+28], ia14_6=A6[i+29],
                              ra15_6=A6[i+30], ia15_6=A6[i+31], ra16_6=A6[i+32],
                              ia16_6=A6[i+33], ra17_6=A6[i+34], ia17_6=A6[i+35],
                              ra18_6=A6[i+36], ia18_6=A6[i+37], ra19_6=A6[i+38],
                              ia19_6=A6[i+39], ra20_6=A6[i+40], ia20_6=A6[i+41],
                              ra21_6=A6[i+42], ia21_6=A6[i+43], ra22_6=A6[i+44],
                              ia22_6=A6[i+45], ra23_6=A6[i+46], ia23_6=A6[i+47],
                              ra0_7=A7[i+0], ia0_7=A7[i+1], ra1_7=A7[i+2],
                              ia1_7=A7[i+3], ra2_7=A7[i+4], ia2_7=A7[i+5],
                              ra3_7=A7[i+6], ia3_7=A7[i+7], ra4_7=A7[i+8],
                              ia4_7=A7[i+9], ra5_7=A7[i+10], ia5_7=A7[i+11],
                              ra6_7=A7[i+12], ia6_7=A7[i+13], ra7_7=A7[i+14],
                              ia7_7=A7[i+15], ra8_7=A7[i+16], ia8_7=A7[i+17],
                              ra9_7=A7[i+18], ia9_7=A7[i+19], ra10_7=A7[i+20],
                              ia10_7=A7[i+21], ra11_7=A7[i+22], ia11_7=A7[i+23],
                              ra12_7=A7[i+24], ia12_7=A7[i+25], ra13_7=A7[i+26],
                              ia13_7=A7[i+27], ra14_7=A7[i+28], ia14_7=A7[i+29],
                              ra15_7=A7[i+30], ia15_7=A7[i+31], ra16_7=A7[i+32],
                              ia16_7=A7[i+33], ra17_7=A7[i+34], ia17_7=A7[i+35],
                              ra18_7=A7[i+36], ia18_7=A7[i+37], ra19_7=A7[i+38],
                              ia19_7=A7[i+39], ra20_7=A7[i+40], ia20_7=A7[i+41],
                              ra21_7=A7[i+42], ia21_7=A7[i+43], ra22_7=A7[i+44],
                              ia22_7=A7[i+45], ra23_7=A7[i+46], ia23_7=A7[i+47],
                              ra0_8=A8[i+0], ia0_8=A8[i+1], ra1_8=A8[i+2],
                              ia1_8=A8[i+3], ra2_8=A8[i+4], ia2_8=A8[i+5],
                              ra3_8=A8[i+6], ia3_8=A8[i+7], ra4_8=A8[i+8],
                              ia4_8=A8[i+9], ra5_8=A8[i+10], ia5_8=A8[i+11],
                              ra6_8=A8[i+12], ia6_8=A8[i+13], ra7_8=A8[i+14],
                              ia7_8=A8[i+15], ra8_8=A8[i+16], ia8_8=A8[i+17],
                              ra9_8=A8[i+18], ia9_8=A8[i+19], ra10_8=A8[i+20],
                              ia10_8=A8[i+21], ra11_8=A8[i+22], ia11_8=A8[i+23],
                              ra12_8=A8[i+24], ia12_8=A8[i+25], ra13_8=A8[i+26],
                              ia13_8=A8[i+27], ra14_8=A8[i+28], ia14_8=A8[i+29],
                              ra15_8=A8[i+30], ia15_8=A8[i+31], ra16_8=A8[i+32],
                              ia16_8=A8[i+33], ra17_8=A8[i+34], ia17_8=A8[i+35],
                              ra18_8=A8[i+36], ia18_8=A8[i+37], ra19_8=A8[i+38],
                              ia19_8=A8[i+39], ra20_8=A8[i+40], ia20_8=A8[i+41],
                              ra21_8=A8[i+42], ia21_8=A8[i+43], ra22_8=A8[i+44],
                              ia22_8=A8[i+45], ra23_8=A8[i+46], ia23_8=A8[i+47],
                              ra0_9=A9[i+0], ia0_9=A9[i+1], ra1_9=A9[i+2],
                              ia1_9=A9[i+3], ra2_9=A9[i+4], ia2_9=A9[i+5],
                              ra3_9=A9[i+6], ia3_9=A9[i+7], ra4_9=A9[i+8],
                              ia4_9=A9[i+9], ra5_9=A9[i+10], ia5_9=A9[i+11],
                              ra6_9=A9[i+12], ia6_9=A9[i+13], ra7_9=A9[i+14],
                              ia7_9=A9[i+15], ra8_9=A9[i+16], ia8_9=A9[i+17],
                              ra9_9=A9[i+18], ia9_9=A9[i+19], ra10_9=A9[i+20],
                              ia10_9=A9[i+21], ra11_9=A9[i+22], ia11_9=A9[i+23],
                              ra12_9=A9[i+24], ia12_9=A9[i+25], ra13_9=A9[i+26],
                              ia13_9=A9[i+27], ra14_9=A9[i+28], ia14_9=A9[i+29],
                              ra15_9=A9[i+30], ia15_9=A9[i+31], ra16_9=A9[i+32],
                              ia16_9=A9[i+33], ra17_9=A9[i+34], ia17_9=A9[i+35],
                              ra18_9=A9[i+36], ia18_9=A9[i+37], ra19_9=A9[i+38],
                              ia19_9=A9[i+39], ra20_9=A9[i+40], ia20_9=A9[i+41],
                              ra21_9=A9[i+42], ia21_9=A9[i+43], ra22_9=A9[i+44],
                              ia22_9=A9[i+45], ra23_9=A9[i+46], ia23_9=A9[i+47],
                              ra0_10=A10[i+0], ia0_10=A10[i+1], ra1_10=A10[i+2],
                              ia1_10=A10[i+3], ra2_10=A10[i+4], ia2_10=A10[i+5],
                              ra3_10=A10[i+6], ia3_10=A10[i+7], ra4_10=A10[i+8],
                              ia4_10=A10[i+9], ra5_10=A10[i+10],
                              ia5_10=A10[i+11], ra6_10=A10[i+12],
                              ia6_10=A10[i+13], ra7_10=A10[i+14],
                              ia7_10=A10[i+15], ra8_10=A10[i+16],
                              ia8_10=A10[i+17], ra9_10=A10[i+18],
                              ia9_10=A10[i+19], ra10_10=A10[i+20],
                              ia10_10=A10[i+21], ra11_10=A10[i+22],
                              ia11_10=A10[i+23], ra12_10=A10[i+24],
                              ia12_10=A10[i+25], ra13_10=A10[i+26],
                              ia13_10=A10[i+27], ra14_10=A10[i+28],
                              ia14_10=A10[i+29], ra15_10=A10[i+30],
                              ia15_10=A10[i+31], ra16_10=A10[i+32],
                              ia16_10=A10[i+33], ra17_10=A10[i+34],
                              ia17_10=A10[i+35], ra18_10=A10[i+36],
                              ia18_10=A10[i+37], ra19_10=A10[i+38],
                              ia19_10=A10[i+39], ra20_10=A10[i+40],
                              ia20_10=A10[i+41], ra21_10=A10[i+42],
                              ia21_10=A10[i+43], ra22_10=A10[i+44],
                              ia22_10=A10[i+45], ra23_10=A10[i+46],
                              ia23_10=A10[i+47], ra0_11=A11[i+0],
                              ia0_11=A11[i+1], ra1_11=A11[i+2], ia1_11=A11[i+3],
                              ra2_11=A11[i+4], ia2_11=A11[i+5], ra3_11=A11[i+6],
                              ia3_11=A11[i+7], ra4_11=A11[i+8], ia4_11=A11[i+9],
                              ra5_11=A11[i+10], ia5_11=A11[i+11],
                              ra6_11=A11[i+12], ia6_11=A11[i+13],
                              ra7_11=A11[i+14], ia7_11=A11[i+15],
                              ra8_11=A11[i+16], ia8_11=A11[i+17],
                              ra9_11=A11[i+18], ia9_11=A11[i+19],
                              ra10_11=A11[i+20], ia10_11=A11[i+21],
                              ra11_11=A11[i+22], ia11_11=A11[i+23],
                              ra12_11=A11[i+24], ia12_11=A11[i+25],
                              ra13_11=A11[i+26], ia13_11=A11[i+27],
                              ra14_11=A11[i+28], ia14_11=A11[i+29],
                              ra15_11=A11[i+30], ia15_11=A11[i+31],
                              ra16_11=A11[i+32], ia16_11=A11[i+33],
                              ra17_11=A11[i+34], ia17_11=A11[i+35],
                              ra18_11=A11[i+36], ia18_11=A11[i+37],
                              ra19_11=A11[i+38], ia19_11=A11[i+39],
                              ra20_11=A11[i+40], ia20_11=A11[i+41],
                              ra21_11=A11[i+42], ia21_11=A11[i+43],
                              ra22_11=A11[i+44], ia22_11=A11[i+45],
                              ra23_11=A11[i+46], ia23_11=A11[i+47];
         const register float rx0=X[i+0], ix0=X[i+1], rx1=X[i+2], ix1=X[i+3],
                              rx2=X[i+4], ix2=X[i+5], rx3=X[i+6], ix3=X[i+7],
                              rx4=X[i+8], ix4=X[i+9], rx5=X[i+10], ix5=X[i+11],
                              rx6=X[i+12], ix6=X[i+13], rx7=X[i+14],
                              ix7=X[i+15], rx8=X[i+16], ix8=X[i+17],
                              rx9=X[i+18], ix9=X[i+19], rx10=X[i+20],
                              ix10=X[i+21], rx11=X[i+22], ix11=X[i+23],
                              rx12=X[i+24], ix12=X[i+25], rx13=X[i+26],
                              ix13=X[i+27], rx14=X[i+28], ix14=X[i+29],
                              rx15=X[i+30], ix15=X[i+31], rx16=X[i+32],
                              ix16=X[i+33], rx17=X[i+34], ix17=X[i+35],
                              rx18=X[i+36], ix18=X[i+37], rx19=X[i+38],
                              ix19=X[i+39], rx20=X[i+40], ix20=X[i+41],
                              rx21=X[i+42], ix21=X[i+43], rx22=X[i+44],
                              ix22=X[i+45], rx23=X[i+46], ix23=X[i+47];

         prefX(X);
         ry0 += ra0_0 * rx0;
         iy0 += ra0_0 * ix0;
         ry1 += ra0_1 * rx0;
         iy1 += ra0_1 * ix0;
         ry2 += ra0_2 * rx0;
         iy2 += ra0_2 * ix0;
         ry3 += ra0_3 * rx0;
         iy3 += ra0_3 * ix0;
         ry4 += ra0_4 * rx0;
         iy4 += ra0_4 * ix0;
         ry5 += ra0_5 * rx0;
         iy5 += ra0_5 * ix0;
         ry6 += ra0_6 * rx0;
         iy6 += ra0_6 * ix0;
         ry7 += ra0_7 * rx0;
         iy7 += ra0_7 * ix0;
         ry8 += ra0_8 * rx0;
         iy8 += ra0_8 * ix0;
         ry9 += ra0_9 * rx0;
         iy9 += ra0_9 * ix0;
         ry10 += ra0_10 * rx0;
         iy10 += ra0_10 * ix0;
         ry11 += ra0_11 * rx0;
         iy11 += ra0_11 * ix0;
         ry0 += ra1_0 * rx1;
         iy0 += ra1_0 * ix1;
         ry1 += ra1_1 * rx1;
         iy1 += ra1_1 * ix1;
         ry2 += ra1_2 * rx1;
         iy2 += ra1_2 * ix1;
         ry3 += ra1_3 * rx1;
         iy3 += ra1_3 * ix1;
         ry4 += ra1_4 * rx1;
         iy4 += ra1_4 * ix1;
         ry5 += ra1_5 * rx1;
         iy5 += ra1_5 * ix1;
         ry6 += ra1_6 * rx1;
         iy6 += ra1_6 * ix1;
         ry7 += ra1_7 * rx1;
         iy7 += ra1_7 * ix1;
         ry8 += ra1_8 * rx1;
         iy8 += ra1_8 * ix1;
         ry9 += ra1_9 * rx1;
         iy9 += ra1_9 * ix1;
         ry10 += ra1_10 * rx1;
         iy10 += ra1_10 * ix1;
         ry11 += ra1_11 * rx1;
         iy11 += ra1_11 * ix1;
         ry0 += ra2_0 * rx2;
         iy0 += ra2_0 * ix2;
         ry1 += ra2_1 * rx2;
         iy1 += ra2_1 * ix2;
         ry2 += ra2_2 * rx2;
         iy2 += ra2_2 * ix2;
         ry3 += ra2_3 * rx2;
         iy3 += ra2_3 * ix2;
         ry4 += ra2_4 * rx2;
         iy4 += ra2_4 * ix2;
         ry5 += ra2_5 * rx2;
         iy5 += ra2_5 * ix2;
         ry6 += ra2_6 * rx2;
         iy6 += ra2_6 * ix2;
         ry7 += ra2_7 * rx2;
         iy7 += ra2_7 * ix2;
         ry8 += ra2_8 * rx2;
         iy8 += ra2_8 * ix2;
         ry9 += ra2_9 * rx2;
         iy9 += ra2_9 * ix2;
         ry10 += ra2_10 * rx2;
         iy10 += ra2_10 * ix2;
         ry11 += ra2_11 * rx2;
         iy11 += ra2_11 * ix2;
         ry0 += ra3_0 * rx3;
         iy0 += ra3_0 * ix3;
         ry1 += ra3_1 * rx3;
         iy1 += ra3_1 * ix3;
         ry2 += ra3_2 * rx3;
         iy2 += ra3_2 * ix3;
         ry3 += ra3_3 * rx3;
         iy3 += ra3_3 * ix3;
         ry4 += ra3_4 * rx3;
         iy4 += ra3_4 * ix3;
         ry5 += ra3_5 * rx3;
         iy5 += ra3_5 * ix3;
         ry6 += ra3_6 * rx3;
         iy6 += ra3_6 * ix3;
         ry7 += ra3_7 * rx3;
         iy7 += ra3_7 * ix3;
         ry8 += ra3_8 * rx3;
         iy8 += ra3_8 * ix3;
         ry9 += ra3_9 * rx3;
         iy9 += ra3_9 * ix3;
         ry10 += ra3_10 * rx3;
         iy10 += ra3_10 * ix3;
         ry11 += ra3_11 * rx3;
         iy11 += ra3_11 * ix3;
         ry0 += ra4_0 * rx4;
         iy0 += ra4_0 * ix4;
         ry1 += ra4_1 * rx4;
         iy1 += ra4_1 * ix4;
         ry2 += ra4_2 * rx4;
         iy2 += ra4_2 * ix4;
         ry3 += ra4_3 * rx4;
         iy3 += ra4_3 * ix4;
         ry4 += ra4_4 * rx4;
         iy4 += ra4_4 * ix4;
         ry5 += ra4_5 * rx4;
         iy5 += ra4_5 * ix4;
         ry6 += ra4_6 * rx4;
         iy6 += ra4_6 * ix4;
         ry7 += ra4_7 * rx4;
         iy7 += ra4_7 * ix4;
         ry8 += ra4_8 * rx4;
         iy8 += ra4_8 * ix4;
         ry9 += ra4_9 * rx4;
         iy9 += ra4_9 * ix4;
         ry10 += ra4_10 * rx4;
         iy10 += ra4_10 * ix4;
         ry11 += ra4_11 * rx4;
         iy11 += ra4_11 * ix4;
         ry0 += ra5_0 * rx5;
         iy0 += ra5_0 * ix5;
         ry1 += ra5_1 * rx5;
         iy1 += ra5_1 * ix5;
         ry2 += ra5_2 * rx5;
         iy2 += ra5_2 * ix5;
         ry3 += ra5_3 * rx5;
         iy3 += ra5_3 * ix5;
         ry4 += ra5_4 * rx5;
         iy4 += ra5_4 * ix5;
         ry5 += ra5_5 * rx5;
         iy5 += ra5_5 * ix5;
         ry6 += ra5_6 * rx5;
         iy6 += ra5_6 * ix5;
         ry7 += ra5_7 * rx5;
         iy7 += ra5_7 * ix5;
         ry8 += ra5_8 * rx5;
         iy8 += ra5_8 * ix5;
         ry9 += ra5_9 * rx5;
         iy9 += ra5_9 * ix5;
         ry10 += ra5_10 * rx5;
         iy10 += ra5_10 * ix5;
         ry11 += ra5_11 * rx5;
         iy11 += ra5_11 * ix5;
         ry0 += ra6_0 * rx6;
         iy0 += ra6_0 * ix6;
         ry1 += ra6_1 * rx6;
         iy1 += ra6_1 * ix6;
         ry2 += ra6_2 * rx6;
         iy2 += ra6_2 * ix6;
         ry3 += ra6_3 * rx6;
         iy3 += ra6_3 * ix6;
         ry4 += ra6_4 * rx6;
         iy4 += ra6_4 * ix6;
         ry5 += ra6_5 * rx6;
         iy5 += ra6_5 * ix6;
         ry6 += ra6_6 * rx6;
         iy6 += ra6_6 * ix6;
         ry7 += ra6_7 * rx6;
         iy7 += ra6_7 * ix6;
         ry8 += ra6_8 * rx6;
         iy8 += ra6_8 * ix6;
         ry9 += ra6_9 * rx6;
         iy9 += ra6_9 * ix6;
         ry10 += ra6_10 * rx6;
         iy10 += ra6_10 * ix6;
         ry11 += ra6_11 * rx6;
         iy11 += ra6_11 * ix6;
         ry0 += ra7_0 * rx7;
         iy0 += ra7_0 * ix7;
         ry1 += ra7_1 * rx7;
         iy1 += ra7_1 * ix7;
         ry2 += ra7_2 * rx7;
         iy2 += ra7_2 * ix7;
         ry3 += ra7_3 * rx7;
         iy3 += ra7_3 * ix7;
         ry4 += ra7_4 * rx7;
         iy4 += ra7_4 * ix7;
         ry5 += ra7_5 * rx7;
         iy5 += ra7_5 * ix7;
         ry6 += ra7_6 * rx7;
         iy6 += ra7_6 * ix7;
         ry7 += ra7_7 * rx7;
         iy7 += ra7_7 * ix7;
         ry8 += ra7_8 * rx7;
         iy8 += ra7_8 * ix7;
         ry9 += ra7_9 * rx7;
         iy9 += ra7_9 * ix7;
         ry10 += ra7_10 * rx7;
         iy10 += ra7_10 * ix7;
         ry11 += ra7_11 * rx7;
         iy11 += ra7_11 * ix7;
         ry0 += ra8_0 * rx8;
         iy0 += ra8_0 * ix8;
         ry1 += ra8_1 * rx8;
         iy1 += ra8_1 * ix8;
         ry2 += ra8_2 * rx8;
         iy2 += ra8_2 * ix8;
         ry3 += ra8_3 * rx8;
         iy3 += ra8_3 * ix8;
         ry4 += ra8_4 * rx8;
         iy4 += ra8_4 * ix8;
         ry5 += ra8_5 * rx8;
         iy5 += ra8_5 * ix8;
         ry6 += ra8_6 * rx8;
         iy6 += ra8_6 * ix8;
         ry7 += ra8_7 * rx8;
         iy7 += ra8_7 * ix8;
         ry8 += ra8_8 * rx8;
         iy8 += ra8_8 * ix8;
         ry9 += ra8_9 * rx8;
         iy9 += ra8_9 * ix8;
         ry10 += ra8_10 * rx8;
         iy10 += ra8_10 * ix8;
         ry11 += ra8_11 * rx8;
         iy11 += ra8_11 * ix8;
         ry0 += ra9_0 * rx9;
         iy0 += ra9_0 * ix9;
         ry1 += ra9_1 * rx9;
         iy1 += ra9_1 * ix9;
         ry2 += ra9_2 * rx9;
         iy2 += ra9_2 * ix9;
         ry3 += ra9_3 * rx9;
         iy3 += ra9_3 * ix9;
         ry4 += ra9_4 * rx9;
         iy4 += ra9_4 * ix9;
         ry5 += ra9_5 * rx9;
         iy5 += ra9_5 * ix9;
         ry6 += ra9_6 * rx9;
         iy6 += ra9_6 * ix9;
         ry7 += ra9_7 * rx9;
         iy7 += ra9_7 * ix9;
         ry8 += ra9_8 * rx9;
         iy8 += ra9_8 * ix9;
         ry9 += ra9_9 * rx9;
         iy9 += ra9_9 * ix9;
         ry10 += ra9_10 * rx9;
         iy10 += ra9_10 * ix9;
         ry11 += ra9_11 * rx9;
         iy11 += ra9_11 * ix9;
         ry0 += ra10_0 * rx10;
         iy0 += ra10_0 * ix10;
         ry1 += ra10_1 * rx10;
         iy1 += ra10_1 * ix10;
         ry2 += ra10_2 * rx10;
         iy2 += ra10_2 * ix10;
         ry3 += ra10_3 * rx10;
         iy3 += ra10_3 * ix10;
         ry4 += ra10_4 * rx10;
         iy4 += ra10_4 * ix10;
         ry5 += ra10_5 * rx10;
         iy5 += ra10_5 * ix10;
         ry6 += ra10_6 * rx10;
         iy6 += ra10_6 * ix10;
         ry7 += ra10_7 * rx10;
         iy7 += ra10_7 * ix10;
         ry8 += ra10_8 * rx10;
         iy8 += ra10_8 * ix10;
         ry9 += ra10_9 * rx10;
         iy9 += ra10_9 * ix10;
         ry10 += ra10_10 * rx10;
         iy10 += ra10_10 * ix10;
         ry11 += ra10_11 * rx10;
         iy11 += ra10_11 * ix10;
         ry0 += ra11_0 * rx11;
         iy0 += ra11_0 * ix11;
         ry1 += ra11_1 * rx11;
         iy1 += ra11_1 * ix11;
         ry2 += ra11_2 * rx11;
         iy2 += ra11_2 * ix11;
         ry3 += ra11_3 * rx11;
         iy3 += ra11_3 * ix11;
         ry4 += ra11_4 * rx11;
         iy4 += ra11_4 * ix11;
         ry5 += ra11_5 * rx11;
         iy5 += ra11_5 * ix11;
         ry6 += ra11_6 * rx11;
         iy6 += ra11_6 * ix11;
         ry7 += ra11_7 * rx11;
         iy7 += ra11_7 * ix11;
         ry8 += ra11_8 * rx11;
         iy8 += ra11_8 * ix11;
         ry9 += ra11_9 * rx11;
         iy9 += ra11_9 * ix11;
         ry10 += ra11_10 * rx11;
         iy10 += ra11_10 * ix11;
         ry11 += ra11_11 * rx11;
         iy11 += ra11_11 * ix11;
         ry0 += ra12_0 * rx12;
         iy0 += ra12_0 * ix12;
         ry1 += ra12_1 * rx12;
         iy1 += ra12_1 * ix12;
         ry2 += ra12_2 * rx12;
         iy2 += ra12_2 * ix12;
         ry3 += ra12_3 * rx12;
         iy3 += ra12_3 * ix12;
         ry4 += ra12_4 * rx12;
         iy4 += ra12_4 * ix12;
         ry5 += ra12_5 * rx12;
         iy5 += ra12_5 * ix12;
         ry6 += ra12_6 * rx12;
         iy6 += ra12_6 * ix12;
         ry7 += ra12_7 * rx12;
         iy7 += ra12_7 * ix12;
         ry8 += ra12_8 * rx12;
         iy8 += ra12_8 * ix12;
         ry9 += ra12_9 * rx12;
         iy9 += ra12_9 * ix12;
         ry10 += ra12_10 * rx12;
         iy10 += ra12_10 * ix12;
         ry11 += ra12_11 * rx12;
         iy11 += ra12_11 * ix12;
         ry0 += ra13_0 * rx13;
         iy0 += ra13_0 * ix13;
         ry1 += ra13_1 * rx13;
         iy1 += ra13_1 * ix13;
         ry2 += ra13_2 * rx13;
         iy2 += ra13_2 * ix13;
         ry3 += ra13_3 * rx13;
         iy3 += ra13_3 * ix13;
         ry4 += ra13_4 * rx13;
         iy4 += ra13_4 * ix13;
         ry5 += ra13_5 * rx13;
         iy5 += ra13_5 * ix13;
         ry6 += ra13_6 * rx13;
         iy6 += ra13_6 * ix13;
         ry7 += ra13_7 * rx13;
         iy7 += ra13_7 * ix13;
         ry8 += ra13_8 * rx13;
         iy8 += ra13_8 * ix13;
         ry9 += ra13_9 * rx13;
         iy9 += ra13_9 * ix13;
         ry10 += ra13_10 * rx13;
         iy10 += ra13_10 * ix13;
         ry11 += ra13_11 * rx13;
         iy11 += ra13_11 * ix13;
         ry0 += ra14_0 * rx14;
         iy0 += ra14_0 * ix14;
         ry1 += ra14_1 * rx14;
         iy1 += ra14_1 * ix14;
         ry2 += ra14_2 * rx14;
         iy2 += ra14_2 * ix14;
         ry3 += ra14_3 * rx14;
         iy3 += ra14_3 * ix14;
         ry4 += ra14_4 * rx14;
         iy4 += ra14_4 * ix14;
         ry5 += ra14_5 * rx14;
         iy5 += ra14_5 * ix14;
         ry6 += ra14_6 * rx14;
         iy6 += ra14_6 * ix14;
         ry7 += ra14_7 * rx14;
         iy7 += ra14_7 * ix14;
         ry8 += ra14_8 * rx14;
         iy8 += ra14_8 * ix14;
         ry9 += ra14_9 * rx14;
         iy9 += ra14_9 * ix14;
         ry10 += ra14_10 * rx14;
         iy10 += ra14_10 * ix14;
         ry11 += ra14_11 * rx14;
         iy11 += ra14_11 * ix14;
         ry0 += ra15_0 * rx15;
         iy0 += ra15_0 * ix15;
         ry1 += ra15_1 * rx15;
         iy1 += ra15_1 * ix15;
         ry2 += ra15_2 * rx15;
         iy2 += ra15_2 * ix15;
         ry3 += ra15_3 * rx15;
         iy3 += ra15_3 * ix15;
         ry4 += ra15_4 * rx15;
         iy4 += ra15_4 * ix15;
         ry5 += ra15_5 * rx15;
         iy5 += ra15_5 * ix15;
         ry6 += ra15_6 * rx15;
         iy6 += ra15_6 * ix15;
         ry7 += ra15_7 * rx15;
         iy7 += ra15_7 * ix15;
         ry8 += ra15_8 * rx15;
         iy8 += ra15_8 * ix15;
         ry9 += ra15_9 * rx15;
         iy9 += ra15_9 * ix15;
         ry10 += ra15_10 * rx15;
         iy10 += ra15_10 * ix15;
         ry11 += ra15_11 * rx15;
         iy11 += ra15_11 * ix15;
         ry0 += ra16_0 * rx16;
         iy0 += ra16_0 * ix16;
         ry1 += ra16_1 * rx16;
         iy1 += ra16_1 * ix16;
         ry2 += ra16_2 * rx16;
         iy2 += ra16_2 * ix16;
         ry3 += ra16_3 * rx16;
         iy3 += ra16_3 * ix16;
         ry4 += ra16_4 * rx16;
         iy4 += ra16_4 * ix16;
         ry5 += ra16_5 * rx16;
         iy5 += ra16_5 * ix16;
         ry6 += ra16_6 * rx16;
         iy6 += ra16_6 * ix16;
         ry7 += ra16_7 * rx16;
         iy7 += ra16_7 * ix16;
         ry8 += ra16_8 * rx16;
         iy8 += ra16_8 * ix16;
         ry9 += ra16_9 * rx16;
         iy9 += ra16_9 * ix16;
         ry10 += ra16_10 * rx16;
         iy10 += ra16_10 * ix16;
         ry11 += ra16_11 * rx16;
         iy11 += ra16_11 * ix16;
         ry0 += ra17_0 * rx17;
         iy0 += ra17_0 * ix17;
         ry1 += ra17_1 * rx17;
         iy1 += ra17_1 * ix17;
         ry2 += ra17_2 * rx17;
         iy2 += ra17_2 * ix17;
         ry3 += ra17_3 * rx17;
         iy3 += ra17_3 * ix17;
         ry4 += ra17_4 * rx17;
         iy4 += ra17_4 * ix17;
         ry5 += ra17_5 * rx17;
         iy5 += ra17_5 * ix17;
         ry6 += ra17_6 * rx17;
         iy6 += ra17_6 * ix17;
         ry7 += ra17_7 * rx17;
         iy7 += ra17_7 * ix17;
         ry8 += ra17_8 * rx17;
         iy8 += ra17_8 * ix17;
         ry9 += ra17_9 * rx17;
         iy9 += ra17_9 * ix17;
         ry10 += ra17_10 * rx17;
         iy10 += ra17_10 * ix17;
         ry11 += ra17_11 * rx17;
         iy11 += ra17_11 * ix17;
         ry0 += ra18_0 * rx18;
         iy0 += ra18_0 * ix18;
         ry1 += ra18_1 * rx18;
         iy1 += ra18_1 * ix18;
         ry2 += ra18_2 * rx18;
         iy2 += ra18_2 * ix18;
         ry3 += ra18_3 * rx18;
         iy3 += ra18_3 * ix18;
         ry4 += ra18_4 * rx18;
         iy4 += ra18_4 * ix18;
         ry5 += ra18_5 * rx18;
         iy5 += ra18_5 * ix18;
         ry6 += ra18_6 * rx18;
         iy6 += ra18_6 * ix18;
         ry7 += ra18_7 * rx18;
         iy7 += ra18_7 * ix18;
         ry8 += ra18_8 * rx18;
         iy8 += ra18_8 * ix18;
         ry9 += ra18_9 * rx18;
         iy9 += ra18_9 * ix18;
         ry10 += ra18_10 * rx18;
         iy10 += ra18_10 * ix18;
         ry11 += ra18_11 * rx18;
         iy11 += ra18_11 * ix18;
         ry0 += ra19_0 * rx19;
         iy0 += ra19_0 * ix19;
         ry1 += ra19_1 * rx19;
         iy1 += ra19_1 * ix19;
         ry2 += ra19_2 * rx19;
         iy2 += ra19_2 * ix19;
         ry3 += ra19_3 * rx19;
         iy3 += ra19_3 * ix19;
         ry4 += ra19_4 * rx19;
         iy4 += ra19_4 * ix19;
         ry5 += ra19_5 * rx19;
         iy5 += ra19_5 * ix19;
         ry6 += ra19_6 * rx19;
         iy6 += ra19_6 * ix19;
         ry7 += ra19_7 * rx19;
         iy7 += ra19_7 * ix19;
         ry8 += ra19_8 * rx19;
         iy8 += ra19_8 * ix19;
         ry9 += ra19_9 * rx19;
         iy9 += ra19_9 * ix19;
         ry10 += ra19_10 * rx19;
         iy10 += ra19_10 * ix19;
         ry11 += ra19_11 * rx19;
         iy11 += ra19_11 * ix19;
         ry0 += ra20_0 * rx20;
         iy0 += ra20_0 * ix20;
         ry1 += ra20_1 * rx20;
         iy1 += ra20_1 * ix20;
         ry2 += ra20_2 * rx20;
         iy2 += ra20_2 * ix20;
         ry3 += ra20_3 * rx20;
         iy3 += ra20_3 * ix20;
         ry4 += ra20_4 * rx20;
         iy4 += ra20_4 * ix20;
         ry5 += ra20_5 * rx20;
         iy5 += ra20_5 * ix20;
         ry6 += ra20_6 * rx20;
         iy6 += ra20_6 * ix20;
         ry7 += ra20_7 * rx20;
         iy7 += ra20_7 * ix20;
         ry8 += ra20_8 * rx20;
         iy8 += ra20_8 * ix20;
         ry9 += ra20_9 * rx20;
         iy9 += ra20_9 * ix20;
         ry10 += ra20_10 * rx20;
         iy10 += ra20_10 * ix20;
         ry11 += ra20_11 * rx20;
         iy11 += ra20_11 * ix20;
         ry0 += ra21_0 * rx21;
         iy0 += ra21_0 * ix21;
         ry1 += ra21_1 * rx21;
         iy1 += ra21_1 * ix21;
         ry2 += ra21_2 * rx21;
         iy2 += ra21_2 * ix21;
         ry3 += ra21_3 * rx21;
         iy3 += ra21_3 * ix21;
         ry4 += ra21_4 * rx21;
         iy4 += ra21_4 * ix21;
         ry5 += ra21_5 * rx21;
         iy5 += ra21_5 * ix21;
         ry6 += ra21_6 * rx21;
         iy6 += ra21_6 * ix21;
         ry7 += ra21_7 * rx21;
         iy7 += ra21_7 * ix21;
         ry8 += ra21_8 * rx21;
         iy8 += ra21_8 * ix21;
         ry9 += ra21_9 * rx21;
         iy9 += ra21_9 * ix21;
         ry10 += ra21_10 * rx21;
         iy10 += ra21_10 * ix21;
         ry11 += ra21_11 * rx21;
         iy11 += ra21_11 * ix21;
         ry0 += ra22_0 * rx22;
         iy0 += ra22_0 * ix22;
         ry1 += ra22_1 * rx22;
         iy1 += ra22_1 * ix22;
         ry2 += ra22_2 * rx22;
         iy2 += ra22_2 * ix22;
         ry3 += ra22_3 * rx22;
         iy3 += ra22_3 * ix22;
         ry4 += ra22_4 * rx22;
         iy4 += ra22_4 * ix22;
         ry5 += ra22_5 * rx22;
         iy5 += ra22_5 * ix22;
         ry6 += ra22_6 * rx22;
         iy6 += ra22_6 * ix22;
         ry7 += ra22_7 * rx22;
         iy7 += ra22_7 * ix22;
         ry8 += ra22_8 * rx22;
         iy8 += ra22_8 * ix22;
         ry9 += ra22_9 * rx22;
         iy9 += ra22_9 * ix22;
         ry10 += ra22_10 * rx22;
         iy10 += ra22_10 * ix22;
         ry11 += ra22_11 * rx22;
         iy11 += ra22_11 * ix22;
         ry0 += ra23_0 * rx23;
         iy0 += ra23_0 * ix23;
         ry1 += ra23_1 * rx23;
         iy1 += ra23_1 * ix23;
         ry2 += ra23_2 * rx23;
         iy2 += ra23_2 * ix23;
         ry3 += ra23_3 * rx23;
         iy3 += ra23_3 * ix23;
         ry4 += ra23_4 * rx23;
         iy4 += ra23_4 * ix23;
         ry5 += ra23_5 * rx23;
         iy5 += ra23_5 * ix23;
         ry6 += ra23_6 * rx23;
         iy6 += ra23_6 * ix23;
         ry7 += ra23_7 * rx23;
         iy7 += ra23_7 * ix23;
         ry8 += ra23_8 * rx23;
         iy8 += ra23_8 * ix23;
         ry9 += ra23_9 * rx23;
         iy9 += ra23_9 * ix23;
         ry10 += ra23_10 * rx23;
         iy10 += ra23_10 * ix23;
         ry11 += ra23_11 * rx23;
         iy11 += ra23_11 * ix23;

         ry0 -= ia0_0 * ix0;
         iy0 += ia0_0 * rx0;
         prefA(A0);
         ry1 -= ia0_1 * ix0;
         iy1 += ia0_1 * rx0;
         prefA(A1);
         ry2 -= ia0_2 * ix0;
         iy2 += ia0_2 * rx0;
         prefA(A2);
         ry3 -= ia0_3 * ix0;
         iy3 += ia0_3 * rx0;
         prefA(A3);
         ry4 -= ia0_4 * ix0;
         iy4 += ia0_4 * rx0;
         prefA(A4);
         ry5 -= ia0_5 * ix0;
         iy5 += ia0_5 * rx0;
         prefA(A5);
         ry6 -= ia0_6 * ix0;
         iy6 += ia0_6 * rx0;
         prefA(A6);
         ry7 -= ia0_7 * ix0;
         iy7 += ia0_7 * rx0;
         prefA(A7);
         ry8 -= ia0_8 * ix0;
         iy8 += ia0_8 * rx0;
         prefA(A8);
         ry9 -= ia0_9 * ix0;
         iy9 += ia0_9 * rx0;
         prefA(A9);
         ry10 -= ia0_10 * ix0;
         iy10 += ia0_10 * rx0;
         prefA(A10);
         ry11 -= ia0_11 * ix0;
         iy11 += ia0_11 * rx0;
         prefA(A11);
         ry0 -= ia1_0 * ix1;
         iy0 += ia1_0 * rx1;
         ry1 -= ia1_1 * ix1;
         iy1 += ia1_1 * rx1;
         ry2 -= ia1_2 * ix1;
         iy2 += ia1_2 * rx1;
         ry3 -= ia1_3 * ix1;
         iy3 += ia1_3 * rx1;
         ry4 -= ia1_4 * ix1;
         iy4 += ia1_4 * rx1;
         ry5 -= ia1_5 * ix1;
         iy5 += ia1_5 * rx1;
         ry6 -= ia1_6 * ix1;
         iy6 += ia1_6 * rx1;
         ry7 -= ia1_7 * ix1;
         iy7 += ia1_7 * rx1;
         ry8 -= ia1_8 * ix1;
         iy8 += ia1_8 * rx1;
         ry9 -= ia1_9 * ix1;
         iy9 += ia1_9 * rx1;
         ry10 -= ia1_10 * ix1;
         iy10 += ia1_10 * rx1;
         ry11 -= ia1_11 * ix1;
         iy11 += ia1_11 * rx1;
         ry0 -= ia2_0 * ix2;
         iy0 += ia2_0 * rx2;
         ry1 -= ia2_1 * ix2;
         iy1 += ia2_1 * rx2;
         ry2 -= ia2_2 * ix2;
         iy2 += ia2_2 * rx2;
         ry3 -= ia2_3 * ix2;
         iy3 += ia2_3 * rx2;
         ry4 -= ia2_4 * ix2;
         iy4 += ia2_4 * rx2;
         ry5 -= ia2_5 * ix2;
         iy5 += ia2_5 * rx2;
         ry6 -= ia2_6 * ix2;
         iy6 += ia2_6 * rx2;
         ry7 -= ia2_7 * ix2;
         iy7 += ia2_7 * rx2;
         ry8 -= ia2_8 * ix2;
         iy8 += ia2_8 * rx2;
         ry9 -= ia2_9 * ix2;
         iy9 += ia2_9 * rx2;
         ry10 -= ia2_10 * ix2;
         iy10 += ia2_10 * rx2;
         ry11 -= ia2_11 * ix2;
         iy11 += ia2_11 * rx2;
         ry0 -= ia3_0 * ix3;
         iy0 += ia3_0 * rx3;
         ry1 -= ia3_1 * ix3;
         iy1 += ia3_1 * rx3;
         ry2 -= ia3_2 * ix3;
         iy2 += ia3_2 * rx3;
         ry3 -= ia3_3 * ix3;
         iy3 += ia3_3 * rx3;
         ry4 -= ia3_4 * ix3;
         iy4 += ia3_4 * rx3;
         ry5 -= ia3_5 * ix3;
         iy5 += ia3_5 * rx3;
         ry6 -= ia3_6 * ix3;
         iy6 += ia3_6 * rx3;
         ry7 -= ia3_7 * ix3;
         iy7 += ia3_7 * rx3;
         ry8 -= ia3_8 * ix3;
         iy8 += ia3_8 * rx3;
         ry9 -= ia3_9 * ix3;
         iy9 += ia3_9 * rx3;
         ry10 -= ia3_10 * ix3;
         iy10 += ia3_10 * rx3;
         ry11 -= ia3_11 * ix3;
         iy11 += ia3_11 * rx3;
         ry0 -= ia4_0 * ix4;
         iy0 += ia4_0 * rx4;
         ry1 -= ia4_1 * ix4;
         iy1 += ia4_1 * rx4;
         ry2 -= ia4_2 * ix4;
         iy2 += ia4_2 * rx4;
         ry3 -= ia4_3 * ix4;
         iy3 += ia4_3 * rx4;
         ry4 -= ia4_4 * ix4;
         iy4 += ia4_4 * rx4;
         ry5 -= ia4_5 * ix4;
         iy5 += ia4_5 * rx4;
         ry6 -= ia4_6 * ix4;
         iy6 += ia4_6 * rx4;
         ry7 -= ia4_7 * ix4;
         iy7 += ia4_7 * rx4;
         ry8 -= ia4_8 * ix4;
         iy8 += ia4_8 * rx4;
         ry9 -= ia4_9 * ix4;
         iy9 += ia4_9 * rx4;
         ry10 -= ia4_10 * ix4;
         iy10 += ia4_10 * rx4;
         ry11 -= ia4_11 * ix4;
         iy11 += ia4_11 * rx4;
         ry0 -= ia5_0 * ix5;
         iy0 += ia5_0 * rx5;
         ry1 -= ia5_1 * ix5;
         iy1 += ia5_1 * rx5;
         ry2 -= ia5_2 * ix5;
         iy2 += ia5_2 * rx5;
         ry3 -= ia5_3 * ix5;
         iy3 += ia5_3 * rx5;
         ry4 -= ia5_4 * ix5;
         iy4 += ia5_4 * rx5;
         ry5 -= ia5_5 * ix5;
         iy5 += ia5_5 * rx5;
         ry6 -= ia5_6 * ix5;
         iy6 += ia5_6 * rx5;
         ry7 -= ia5_7 * ix5;
         iy7 += ia5_7 * rx5;
         ry8 -= ia5_8 * ix5;
         iy8 += ia5_8 * rx5;
         ry9 -= ia5_9 * ix5;
         iy9 += ia5_9 * rx5;
         ry10 -= ia5_10 * ix5;
         iy10 += ia5_10 * rx5;
         ry11 -= ia5_11 * ix5;
         iy11 += ia5_11 * rx5;
         ry0 -= ia6_0 * ix6;
         iy0 += ia6_0 * rx6;
         ry1 -= ia6_1 * ix6;
         iy1 += ia6_1 * rx6;
         ry2 -= ia6_2 * ix6;
         iy2 += ia6_2 * rx6;
         ry3 -= ia6_3 * ix6;
         iy3 += ia6_3 * rx6;
         ry4 -= ia6_4 * ix6;
         iy4 += ia6_4 * rx6;
         ry5 -= ia6_5 * ix6;
         iy5 += ia6_5 * rx6;
         ry6 -= ia6_6 * ix6;
         iy6 += ia6_6 * rx6;
         ry7 -= ia6_7 * ix6;
         iy7 += ia6_7 * rx6;
         ry8 -= ia6_8 * ix6;
         iy8 += ia6_8 * rx6;
         ry9 -= ia6_9 * ix6;
         iy9 += ia6_9 * rx6;
         ry10 -= ia6_10 * ix6;
         iy10 += ia6_10 * rx6;
         ry11 -= ia6_11 * ix6;
         iy11 += ia6_11 * rx6;
         ry0 -= ia7_0 * ix7;
         iy0 += ia7_0 * rx7;
         ry1 -= ia7_1 * ix7;
         iy1 += ia7_1 * rx7;
         ry2 -= ia7_2 * ix7;
         iy2 += ia7_2 * rx7;
         ry3 -= ia7_3 * ix7;
         iy3 += ia7_3 * rx7;
         ry4 -= ia7_4 * ix7;
         iy4 += ia7_4 * rx7;
         ry5 -= ia7_5 * ix7;
         iy5 += ia7_5 * rx7;
         ry6 -= ia7_6 * ix7;
         iy6 += ia7_6 * rx7;
         ry7 -= ia7_7 * ix7;
         iy7 += ia7_7 * rx7;
         ry8 -= ia7_8 * ix7;
         iy8 += ia7_8 * rx7;
         ry9 -= ia7_9 * ix7;
         iy9 += ia7_9 * rx7;
         ry10 -= ia7_10 * ix7;
         iy10 += ia7_10 * rx7;
         ry11 -= ia7_11 * ix7;
         iy11 += ia7_11 * rx7;
         ry0 -= ia8_0 * ix8;
         iy0 += ia8_0 * rx8;
         prefA(A0);
         ry1 -= ia8_1 * ix8;
         iy1 += ia8_1 * rx8;
         prefA(A1);
         ry2 -= ia8_2 * ix8;
         iy2 += ia8_2 * rx8;
         prefA(A2);
         ry3 -= ia8_3 * ix8;
         iy3 += ia8_3 * rx8;
         prefA(A3);
         ry4 -= ia8_4 * ix8;
         iy4 += ia8_4 * rx8;
         prefA(A4);
         ry5 -= ia8_5 * ix8;
         iy5 += ia8_5 * rx8;
         prefA(A5);
         ry6 -= ia8_6 * ix8;
         iy6 += ia8_6 * rx8;
         prefA(A6);
         ry7 -= ia8_7 * ix8;
         iy7 += ia8_7 * rx8;
         prefA(A7);
         ry8 -= ia8_8 * ix8;
         iy8 += ia8_8 * rx8;
         prefA(A8);
         ry9 -= ia8_9 * ix8;
         iy9 += ia8_9 * rx8;
         prefA(A9);
         ry10 -= ia8_10 * ix8;
         iy10 += ia8_10 * rx8;
         prefA(A10);
         ry11 -= ia8_11 * ix8;
         iy11 += ia8_11 * rx8;
         prefA(A11);
         ry0 -= ia9_0 * ix9;
         iy0 += ia9_0 * rx9;
         ry1 -= ia9_1 * ix9;
         iy1 += ia9_1 * rx9;
         ry2 -= ia9_2 * ix9;
         iy2 += ia9_2 * rx9;
         ry3 -= ia9_3 * ix9;
         iy3 += ia9_3 * rx9;
         ry4 -= ia9_4 * ix9;
         iy4 += ia9_4 * rx9;
         ry5 -= ia9_5 * ix9;
         iy5 += ia9_5 * rx9;
         ry6 -= ia9_6 * ix9;
         iy6 += ia9_6 * rx9;
         ry7 -= ia9_7 * ix9;
         iy7 += ia9_7 * rx9;
         ry8 -= ia9_8 * ix9;
         iy8 += ia9_8 * rx9;
         ry9 -= ia9_9 * ix9;
         iy9 += ia9_9 * rx9;
         ry10 -= ia9_10 * ix9;
         iy10 += ia9_10 * rx9;
         ry11 -= ia9_11 * ix9;
         iy11 += ia9_11 * rx9;
         ry0 -= ia10_0 * ix10;
         iy0 += ia10_0 * rx10;
         ry1 -= ia10_1 * ix10;
         iy1 += ia10_1 * rx10;
         ry2 -= ia10_2 * ix10;
         iy2 += ia10_2 * rx10;
         ry3 -= ia10_3 * ix10;
         iy3 += ia10_3 * rx10;
         ry4 -= ia10_4 * ix10;
         iy4 += ia10_4 * rx10;
         ry5 -= ia10_5 * ix10;
         iy5 += ia10_5 * rx10;
         ry6 -= ia10_6 * ix10;
         iy6 += ia10_6 * rx10;
         ry7 -= ia10_7 * ix10;
         iy7 += ia10_7 * rx10;
         ry8 -= ia10_8 * ix10;
         iy8 += ia10_8 * rx10;
         ry9 -= ia10_9 * ix10;
         iy9 += ia10_9 * rx10;
         ry10 -= ia10_10 * ix10;
         iy10 += ia10_10 * rx10;
         ry11 -= ia10_11 * ix10;
         iy11 += ia10_11 * rx10;
         ry0 -= ia11_0 * ix11;
         iy0 += ia11_0 * rx11;
         ry1 -= ia11_1 * ix11;
         iy1 += ia11_1 * rx11;
         ry2 -= ia11_2 * ix11;
         iy2 += ia11_2 * rx11;
         ry3 -= ia11_3 * ix11;
         iy3 += ia11_3 * rx11;
         ry4 -= ia11_4 * ix11;
         iy4 += ia11_4 * rx11;
         ry5 -= ia11_5 * ix11;
         iy5 += ia11_5 * rx11;
         ry6 -= ia11_6 * ix11;
         iy6 += ia11_6 * rx11;
         ry7 -= ia11_7 * ix11;
         iy7 += ia11_7 * rx11;
         ry8 -= ia11_8 * ix11;
         iy8 += ia11_8 * rx11;
         ry9 -= ia11_9 * ix11;
         iy9 += ia11_9 * rx11;
         ry10 -= ia11_10 * ix11;
         iy10 += ia11_10 * rx11;
         ry11 -= ia11_11 * ix11;
         iy11 += ia11_11 * rx11;
         ry0 -= ia12_0 * ix12;
         iy0 += ia12_0 * rx12;
         ry1 -= ia12_1 * ix12;
         iy1 += ia12_1 * rx12;
         ry2 -= ia12_2 * ix12;
         iy2 += ia12_2 * rx12;
         ry3 -= ia12_3 * ix12;
         iy3 += ia12_3 * rx12;
         ry4 -= ia12_4 * ix12;
         iy4 += ia12_4 * rx12;
         ry5 -= ia12_5 * ix12;
         iy5 += ia12_5 * rx12;
         ry6 -= ia12_6 * ix12;
         iy6 += ia12_6 * rx12;
         ry7 -= ia12_7 * ix12;
         iy7 += ia12_7 * rx12;
         ry8 -= ia12_8 * ix12;
         iy8 += ia12_8 * rx12;
         ry9 -= ia12_9 * ix12;
         iy9 += ia12_9 * rx12;
         ry10 -= ia12_10 * ix12;
         iy10 += ia12_10 * rx12;
         ry11 -= ia12_11 * ix12;
         iy11 += ia12_11 * rx12;
         ry0 -= ia13_0 * ix13;
         iy0 += ia13_0 * rx13;
         ry1 -= ia13_1 * ix13;
         iy1 += ia13_1 * rx13;
         ry2 -= ia13_2 * ix13;
         iy2 += ia13_2 * rx13;
         ry3 -= ia13_3 * ix13;
         iy3 += ia13_3 * rx13;
         ry4 -= ia13_4 * ix13;
         iy4 += ia13_4 * rx13;
         ry5 -= ia13_5 * ix13;
         iy5 += ia13_5 * rx13;
         ry6 -= ia13_6 * ix13;
         iy6 += ia13_6 * rx13;
         ry7 -= ia13_7 * ix13;
         iy7 += ia13_7 * rx13;
         ry8 -= ia13_8 * ix13;
         iy8 += ia13_8 * rx13;
         ry9 -= ia13_9 * ix13;
         iy9 += ia13_9 * rx13;
         ry10 -= ia13_10 * ix13;
         iy10 += ia13_10 * rx13;
         ry11 -= ia13_11 * ix13;
         iy11 += ia13_11 * rx13;
         ry0 -= ia14_0 * ix14;
         iy0 += ia14_0 * rx14;
         ry1 -= ia14_1 * ix14;
         iy1 += ia14_1 * rx14;
         ry2 -= ia14_2 * ix14;
         iy2 += ia14_2 * rx14;
         ry3 -= ia14_3 * ix14;
         iy3 += ia14_3 * rx14;
         ry4 -= ia14_4 * ix14;
         iy4 += ia14_4 * rx14;
         ry5 -= ia14_5 * ix14;
         iy5 += ia14_5 * rx14;
         ry6 -= ia14_6 * ix14;
         iy6 += ia14_6 * rx14;
         ry7 -= ia14_7 * ix14;
         iy7 += ia14_7 * rx14;
         ry8 -= ia14_8 * ix14;
         iy8 += ia14_8 * rx14;
         ry9 -= ia14_9 * ix14;
         iy9 += ia14_9 * rx14;
         ry10 -= ia14_10 * ix14;
         iy10 += ia14_10 * rx14;
         ry11 -= ia14_11 * ix14;
         iy11 += ia14_11 * rx14;
         ry0 -= ia15_0 * ix15;
         iy0 += ia15_0 * rx15;
         ry1 -= ia15_1 * ix15;
         iy1 += ia15_1 * rx15;
         ry2 -= ia15_2 * ix15;
         iy2 += ia15_2 * rx15;
         ry3 -= ia15_3 * ix15;
         iy3 += ia15_3 * rx15;
         ry4 -= ia15_4 * ix15;
         iy4 += ia15_4 * rx15;
         ry5 -= ia15_5 * ix15;
         iy5 += ia15_5 * rx15;
         ry6 -= ia15_6 * ix15;
         iy6 += ia15_6 * rx15;
         ry7 -= ia15_7 * ix15;
         iy7 += ia15_7 * rx15;
         ry8 -= ia15_8 * ix15;
         iy8 += ia15_8 * rx15;
         ry9 -= ia15_9 * ix15;
         iy9 += ia15_9 * rx15;
         ry10 -= ia15_10 * ix15;
         iy10 += ia15_10 * rx15;
         ry11 -= ia15_11 * ix15;
         iy11 += ia15_11 * rx15;
         ry0 -= ia16_0 * ix16;
         iy0 += ia16_0 * rx16;
         prefA(A0);
         ry1 -= ia16_1 * ix16;
         iy1 += ia16_1 * rx16;
         prefA(A1);
         ry2 -= ia16_2 * ix16;
         iy2 += ia16_2 * rx16;
         prefA(A2);
         ry3 -= ia16_3 * ix16;
         iy3 += ia16_3 * rx16;
         prefA(A3);
         ry4 -= ia16_4 * ix16;
         iy4 += ia16_4 * rx16;
         prefA(A4);
         ry5 -= ia16_5 * ix16;
         iy5 += ia16_5 * rx16;
         prefA(A5);
         ry6 -= ia16_6 * ix16;
         iy6 += ia16_6 * rx16;
         prefA(A6);
         ry7 -= ia16_7 * ix16;
         iy7 += ia16_7 * rx16;
         prefA(A7);
         ry8 -= ia16_8 * ix16;
         iy8 += ia16_8 * rx16;
         prefA(A8);
         ry9 -= ia16_9 * ix16;
         iy9 += ia16_9 * rx16;
         prefA(A9);
         ry10 -= ia16_10 * ix16;
         iy10 += ia16_10 * rx16;
         prefA(A10);
         ry11 -= ia16_11 * ix16;
         iy11 += ia16_11 * rx16;
         prefA(A11);
         ry0 -= ia17_0 * ix17;
         iy0 += ia17_0 * rx17;
         ry1 -= ia17_1 * ix17;
         iy1 += ia17_1 * rx17;
         ry2 -= ia17_2 * ix17;
         iy2 += ia17_2 * rx17;
         ry3 -= ia17_3 * ix17;
         iy3 += ia17_3 * rx17;
         ry4 -= ia17_4 * ix17;
         iy4 += ia17_4 * rx17;
         ry5 -= ia17_5 * ix17;
         iy5 += ia17_5 * rx17;
         ry6 -= ia17_6 * ix17;
         iy6 += ia17_6 * rx17;
         ry7 -= ia17_7 * ix17;
         iy7 += ia17_7 * rx17;
         ry8 -= ia17_8 * ix17;
         iy8 += ia17_8 * rx17;
         ry9 -= ia17_9 * ix17;
         iy9 += ia17_9 * rx17;
         ry10 -= ia17_10 * ix17;
         iy10 += ia17_10 * rx17;
         ry11 -= ia17_11 * ix17;
         iy11 += ia17_11 * rx17;
         ry0 -= ia18_0 * ix18;
         iy0 += ia18_0 * rx18;
         ry1 -= ia18_1 * ix18;
         iy1 += ia18_1 * rx18;
         ry2 -= ia18_2 * ix18;
         iy2 += ia18_2 * rx18;
         ry3 -= ia18_3 * ix18;
         iy3 += ia18_3 * rx18;
         ry4 -= ia18_4 * ix18;
         iy4 += ia18_4 * rx18;
         ry5 -= ia18_5 * ix18;
         iy5 += ia18_5 * rx18;
         ry6 -= ia18_6 * ix18;
         iy6 += ia18_6 * rx18;
         ry7 -= ia18_7 * ix18;
         iy7 += ia18_7 * rx18;
         ry8 -= ia18_8 * ix18;
         iy8 += ia18_8 * rx18;
         ry9 -= ia18_9 * ix18;
         iy9 += ia18_9 * rx18;
         ry10 -= ia18_10 * ix18;
         iy10 += ia18_10 * rx18;
         ry11 -= ia18_11 * ix18;
         iy11 += ia18_11 * rx18;
         ry0 -= ia19_0 * ix19;
         iy0 += ia19_0 * rx19;
         ry1 -= ia19_1 * ix19;
         iy1 += ia19_1 * rx19;
         ry2 -= ia19_2 * ix19;
         iy2 += ia19_2 * rx19;
         ry3 -= ia19_3 * ix19;
         iy3 += ia19_3 * rx19;
         ry4 -= ia19_4 * ix19;
         iy4 += ia19_4 * rx19;
         ry5 -= ia19_5 * ix19;
         iy5 += ia19_5 * rx19;
         ry6 -= ia19_6 * ix19;
         iy6 += ia19_6 * rx19;
         ry7 -= ia19_7 * ix19;
         iy7 += ia19_7 * rx19;
         ry8 -= ia19_8 * ix19;
         iy8 += ia19_8 * rx19;
         ry9 -= ia19_9 * ix19;
         iy9 += ia19_9 * rx19;
         ry10 -= ia19_10 * ix19;
         iy10 += ia19_10 * rx19;
         ry11 -= ia19_11 * ix19;
         iy11 += ia19_11 * rx19;
         ry0 -= ia20_0 * ix20;
         iy0 += ia20_0 * rx20;
         ry1 -= ia20_1 * ix20;
         iy1 += ia20_1 * rx20;
         ry2 -= ia20_2 * ix20;
         iy2 += ia20_2 * rx20;
         ry3 -= ia20_3 * ix20;
         iy3 += ia20_3 * rx20;
         ry4 -= ia20_4 * ix20;
         iy4 += ia20_4 * rx20;
         ry5 -= ia20_5 * ix20;
         iy5 += ia20_5 * rx20;
         ry6 -= ia20_6 * ix20;
         iy6 += ia20_6 * rx20;
         ry7 -= ia20_7 * ix20;
         iy7 += ia20_7 * rx20;
         ry8 -= ia20_8 * ix20;
         iy8 += ia20_8 * rx20;
         ry9 -= ia20_9 * ix20;
         iy9 += ia20_9 * rx20;
         ry10 -= ia20_10 * ix20;
         iy10 += ia20_10 * rx20;
         ry11 -= ia20_11 * ix20;
         iy11 += ia20_11 * rx20;
         ry0 -= ia21_0 * ix21;
         iy0 += ia21_0 * rx21;
         ry1 -= ia21_1 * ix21;
         iy1 += ia21_1 * rx21;
         ry2 -= ia21_2 * ix21;
         iy2 += ia21_2 * rx21;
         ry3 -= ia21_3 * ix21;
         iy3 += ia21_3 * rx21;
         ry4 -= ia21_4 * ix21;
         iy4 += ia21_4 * rx21;
         ry5 -= ia21_5 * ix21;
         iy5 += ia21_5 * rx21;
         ry6 -= ia21_6 * ix21;
         iy6 += ia21_6 * rx21;
         ry7 -= ia21_7 * ix21;
         iy7 += ia21_7 * rx21;
         ry8 -= ia21_8 * ix21;
         iy8 += ia21_8 * rx21;
         ry9 -= ia21_9 * ix21;
         iy9 += ia21_9 * rx21;
         ry10 -= ia21_10 * ix21;
         iy10 += ia21_10 * rx21;
         ry11 -= ia21_11 * ix21;
         iy11 += ia21_11 * rx21;
         ry0 -= ia22_0 * ix22;
         iy0 += ia22_0 * rx22;
         ry1 -= ia22_1 * ix22;
         iy1 += ia22_1 * rx22;
         ry2 -= ia22_2 * ix22;
         iy2 += ia22_2 * rx22;
         ry3 -= ia22_3 * ix22;
         iy3 += ia22_3 * rx22;
         ry4 -= ia22_4 * ix22;
         iy4 += ia22_4 * rx22;
         ry5 -= ia22_5 * ix22;
         iy5 += ia22_5 * rx22;
         ry6 -= ia22_6 * ix22;
         iy6 += ia22_6 * rx22;
         ry7 -= ia22_7 * ix22;
         iy7 += ia22_7 * rx22;
         ry8 -= ia22_8 * ix22;
         iy8 += ia22_8 * rx22;
         ry9 -= ia22_9 * ix22;
         iy9 += ia22_9 * rx22;
         ry10 -= ia22_10 * ix22;
         iy10 += ia22_10 * rx22;
         ry11 -= ia22_11 * ix22;
         iy11 += ia22_11 * rx22;
         ry0 -= ia23_0 * ix23;
         iy0 += ia23_0 * rx23;
         ry1 -= ia23_1 * ix23;
         iy1 += ia23_1 * rx23;
         ry2 -= ia23_2 * ix23;
         iy2 += ia23_2 * rx23;
         ry3 -= ia23_3 * ix23;
         iy3 += ia23_3 * rx23;
         ry4 -= ia23_4 * ix23;
         iy4 += ia23_4 * rx23;
         ry5 -= ia23_5 * ix23;
         iy5 += ia23_5 * rx23;
         ry6 -= ia23_6 * ix23;
         iy6 += ia23_6 * rx23;
         ry7 -= ia23_7 * ix23;
         iy7 += ia23_7 * rx23;
         ry8 -= ia23_8 * ix23;
         iy8 += ia23_8 * rx23;
         ry9 -= ia23_9 * ix23;
         iy9 += ia23_9 * rx23;
         ry10 -= ia23_10 * ix23;
         iy10 += ia23_10 * rx23;
         ry11 -= ia23_11 * ix23;
         iy11 += ia23_11 * rx23;
      }
      for (i=M24; i < M; i += 2)
      {
         const register float rx0 = X[i], ix0 = X[i+1];
         const register float ra0_0=A0[i], ia0_0=A0[i+1], ra0_1=A1[i],
                              ia0_1=A1[i+1], ra0_2=A2[i], ia0_2=A2[i+1],
                              ra0_3=A3[i], ia0_3=A3[i+1], ra0_4=A4[i],
                              ia0_4=A4[i+1], ra0_5=A5[i], ia0_5=A5[i+1],
                              ra0_6=A6[i], ia0_6=A6[i+1], ra0_7=A7[i],
                              ia0_7=A7[i+1], ra0_8=A8[i], ia0_8=A8[i+1],
                              ra0_9=A9[i], ia0_9=A9[i+1], ra0_10=A10[i],
                              ia0_10=A10[i+1], ra0_11=A11[i], ia0_11=A11[i+1];
         ry0 += ra0_0 * rx0;
         iy0 += ra0_0 * ix0;
         ry1 += ra0_1 * rx0;
         iy1 += ra0_1 * ix0;
         ry2 += ra0_2 * rx0;
         iy2 += ra0_2 * ix0;
         ry3 += ra0_3 * rx0;
         iy3 += ra0_3 * ix0;
         ry4 += ra0_4 * rx0;
         iy4 += ra0_4 * ix0;
         ry5 += ra0_5 * rx0;
         iy5 += ra0_5 * ix0;
         ry6 += ra0_6 * rx0;
         iy6 += ra0_6 * ix0;
         ry7 += ra0_7 * rx0;
         iy7 += ra0_7 * ix0;
         ry8 += ra0_8 * rx0;
         iy8 += ra0_8 * ix0;
         ry9 += ra0_9 * rx0;
         iy9 += ra0_9 * ix0;
         ry10 += ra0_10 * rx0;
         iy10 += ra0_10 * ix0;
         ry11 += ra0_11 * rx0;
         iy11 += ra0_11 * ix0;
         ry0 -= ia0_0 * ix0;
         iy0 += ia0_0 * rx0;
         ry1 -= ia0_1 * ix0;
         iy1 += ia0_1 * rx0;
         ry2 -= ia0_2 * ix0;
         iy2 += ia0_2 * rx0;
         ry3 -= ia0_3 * ix0;
         iy3 += ia0_3 * rx0;
         ry4 -= ia0_4 * ix0;
         iy4 += ia0_4 * rx0;
         ry5 -= ia0_5 * ix0;
         iy5 += ia0_5 * rx0;
         ry6 -= ia0_6 * ix0;
         iy6 += ia0_6 * rx0;
         ry7 -= ia0_7 * ix0;
         iy7 += ia0_7 * rx0;
         ry8 -= ia0_8 * ix0;
         iy8 += ia0_8 * rx0;
         ry9 -= ia0_9 * ix0;
         iy9 += ia0_9 * rx0;
         ry10 -= ia0_10 * ix0;
         iy10 += ia0_10 * rx0;
         ry11 -= ia0_11 * ix0;
         iy11 += ia0_11 * rx0;
      }
      #ifdef BETA0
         Y[0] = ry0;
         Y[1] = iy0;
         Y[2] = ry1;
         Y[3] = iy1;
         Y[4] = ry2;
         Y[5] = iy2;
         Y[6] = ry3;
         Y[7] = iy3;
         Y[8] = ry4;
         Y[9] = iy4;
         Y[10] = ry5;
         Y[11] = iy5;
         Y[12] = ry6;
         Y[13] = iy6;
         Y[14] = ry7;
         Y[15] = iy7;
         Y[16] = ry8;
         Y[17] = iy8;
         Y[18] = ry9;
         Y[19] = iy9;
         Y[20] = ry10;
         Y[21] = iy10;
         Y[22] = ry11;
         Y[23] = iy11;
      #else
         Y[0] += ry0;
         Y[1] += iy0;
         Y[2] += ry1;
         Y[3] += iy1;
         Y[4] += ry2;
         Y[5] += iy2;
         Y[6] += ry3;
         Y[7] += iy3;
         Y[8] += ry4;
         Y[9] += iy4;
         Y[10] += ry5;
         Y[11] += iy5;
         Y[12] += ry6;
         Y[13] += iy6;
         Y[14] += ry7;
         Y[15] += iy7;
         Y[16] += ry8;
         Y[17] += iy8;
         Y[18] += ry9;
         Y[19] += iy9;
         Y[20] += ry10;
         Y[21] += iy10;
         Y[22] += ry11;
         Y[23] += iy11;
      #endif
   }
/*
 * Do remaining columns with NU=1 cleanup
 */
   for (j=N-N12; j; j--, A += lda, Y += 2)
   {
      const float *A0=A;
      register float ry0=ATL_rzero, iy0=ATL_rzero;
      ATL_INT i;
      prefY(Y+24+24-1);
      for (i=0; i < M24; i += 48)
      {
         const register float ra0_0=A0[i+0], ia0_0=A0[i+1], ra1_0=A0[i+2],
                              ia1_0=A0[i+3], ra2_0=A0[i+4], ia2_0=A0[i+5],
                              ra3_0=A0[i+6], ia3_0=A0[i+7], ra4_0=A0[i+8],
                              ia4_0=A0[i+9], ra5_0=A0[i+10], ia5_0=A0[i+11],
                              ra6_0=A0[i+12], ia6_0=A0[i+13], ra7_0=A0[i+14],
                              ia7_0=A0[i+15], ra8_0=A0[i+16], ia8_0=A0[i+17],
                              ra9_0=A0[i+18], ia9_0=A0[i+19], ra10_0=A0[i+20],
                              ia10_0=A0[i+21], ra11_0=A0[i+22], ia11_0=A0[i+23],
                              ra12_0=A0[i+24], ia12_0=A0[i+25], ra13_0=A0[i+26],
                              ia13_0=A0[i+27], ra14_0=A0[i+28], ia14_0=A0[i+29],
                              ra15_0=A0[i+30], ia15_0=A0[i+31], ra16_0=A0[i+32],
                              ia16_0=A0[i+33], ra17_0=A0[i+34], ia17_0=A0[i+35],
                              ra18_0=A0[i+36], ia18_0=A0[i+37], ra19_0=A0[i+38],
                              ia19_0=A0[i+39], ra20_0=A0[i+40], ia20_0=A0[i+41],
                              ra21_0=A0[i+42], ia21_0=A0[i+43], ra22_0=A0[i+44],
                              ia22_0=A0[i+45], ra23_0=A0[i+46],
                              ia23_0=A0[i+47];
         const register float rx0=X[i+0], ix0=X[i+1], rx1=X[i+2], ix1=X[i+3],
                              rx2=X[i+4], ix2=X[i+5], rx3=X[i+6], ix3=X[i+7],
                              rx4=X[i+8], ix4=X[i+9], rx5=X[i+10], ix5=X[i+11],
                              rx6=X[i+12], ix6=X[i+13], rx7=X[i+14],
                              ix7=X[i+15], rx8=X[i+16], ix8=X[i+17],
                              rx9=X[i+18], ix9=X[i+19], rx10=X[i+20],
                              ix10=X[i+21], rx11=X[i+22], ix11=X[i+23],
                              rx12=X[i+24], ix12=X[i+25], rx13=X[i+26],
                              ix13=X[i+27], rx14=X[i+28], ix14=X[i+29],
                              rx15=X[i+30], ix15=X[i+31], rx16=X[i+32],
                              ix16=X[i+33], rx17=X[i+34], ix17=X[i+35],
                              rx18=X[i+36], ix18=X[i+37], rx19=X[i+38],
                              ix19=X[i+39], rx20=X[i+40], ix20=X[i+41],
                              rx21=X[i+42], ix21=X[i+43], rx22=X[i+44],
                              ix22=X[i+45], rx23=X[i+46], ix23=X[i+47];

         prefX(X);
         ry0 += ra0_0 * rx0;
         iy0 += ra0_0 * ix0;
         ry0 += ra1_0 * rx1;
         iy0 += ra1_0 * ix1;
         ry0 += ra2_0 * rx2;
         iy0 += ra2_0 * ix2;
         ry0 += ra3_0 * rx3;
         iy0 += ra3_0 * ix3;
         ry0 += ra4_0 * rx4;
         iy0 += ra4_0 * ix4;
         ry0 += ra5_0 * rx5;
         iy0 += ra5_0 * ix5;
         ry0 += ra6_0 * rx6;
         iy0 += ra6_0 * ix6;
         ry0 += ra7_0 * rx7;
         iy0 += ra7_0 * ix7;
         ry0 += ra8_0 * rx8;
         iy0 += ra8_0 * ix8;
         ry0 += ra9_0 * rx9;
         iy0 += ra9_0 * ix9;
         ry0 += ra10_0 * rx10;
         iy0 += ra10_0 * ix10;
         ry0 += ra11_0 * rx11;
         iy0 += ra11_0 * ix11;
         ry0 += ra12_0 * rx12;
         iy0 += ra12_0 * ix12;
         ry0 += ra13_0 * rx13;
         iy0 += ra13_0 * ix13;
         ry0 += ra14_0 * rx14;
         iy0 += ra14_0 * ix14;
         ry0 += ra15_0 * rx15;
         iy0 += ra15_0 * ix15;
         ry0 += ra16_0 * rx16;
         iy0 += ra16_0 * ix16;
         ry0 += ra17_0 * rx17;
         iy0 += ra17_0 * ix17;
         ry0 += ra18_0 * rx18;
         iy0 += ra18_0 * ix18;
         ry0 += ra19_0 * rx19;
         iy0 += ra19_0 * ix19;
         ry0 += ra20_0 * rx20;
         iy0 += ra20_0 * ix20;
         ry0 += ra21_0 * rx21;
         iy0 += ra21_0 * ix21;
         ry0 += ra22_0 * rx22;
         iy0 += ra22_0 * ix22;
         ry0 += ra23_0 * rx23;
         iy0 += ra23_0 * ix23;

         ry0 -= ia0_0 * ix0;
         iy0 += ia0_0 * rx0;
         prefA(A0);
         ry0 -= ia1_0 * ix1;
         iy0 += ia1_0 * rx1;
         ry0 -= ia2_0 * ix2;
         iy0 += ia2_0 * rx2;
         ry0 -= ia3_0 * ix3;
         iy0 += ia3_0 * rx3;
         ry0 -= ia4_0 * ix4;
         iy0 += ia4_0 * rx4;
         ry0 -= ia5_0 * ix5;
         iy0 += ia5_0 * rx5;
         ry0 -= ia6_0 * ix6;
         iy0 += ia6_0 * rx6;
         ry0 -= ia7_0 * ix7;
         iy0 += ia7_0 * rx7;
         ry0 -= ia8_0 * ix8;
         iy0 += ia8_0 * rx8;
         prefA(A0);
         ry0 -= ia9_0 * ix9;
         iy0 += ia9_0 * rx9;
         ry0 -= ia10_0 * ix10;
         iy0 += ia10_0 * rx10;
         ry0 -= ia11_0 * ix11;
         iy0 += ia11_0 * rx11;
         ry0 -= ia12_0 * ix12;
         iy0 += ia12_0 * rx12;
         ry0 -= ia13_0 * ix13;
         iy0 += ia13_0 * rx13;
         ry0 -= ia14_0 * ix14;
         iy0 += ia14_0 * rx14;
         ry0 -= ia15_0 * ix15;
         iy0 += ia15_0 * rx15;
         ry0 -= ia16_0 * ix16;
         iy0 += ia16_0 * rx16;
         prefA(A0);
         ry0 -= ia17_0 * ix17;
         iy0 += ia17_0 * rx17;
         ry0 -= ia18_0 * ix18;
         iy0 += ia18_0 * rx18;
         ry0 -= ia19_0 * ix19;
         iy0 += ia19_0 * rx19;
         ry0 -= ia20_0 * ix20;
         iy0 += ia20_0 * rx20;
         ry0 -= ia21_0 * ix21;
         iy0 += ia21_0 * rx21;
         ry0 -= ia22_0 * ix22;
         iy0 += ia22_0 * rx22;
         ry0 -= ia23_0 * ix23;
         iy0 += ia23_0 * rx23;
      }
      for (i=M24; i < M; i += 2)
      {
         const register float rx0 = X[i], ix0 = X[i+1];
         const register float ra0_0=A0[i], ia0_0=A0[i+1];
         ry0 += ra0_0 * rx0;
         iy0 += ra0_0 * ix0;
         ry0 -= ia0_0 * ix0;
         iy0 += ia0_0 * rx0;
      }
      #ifdef BETA0
         Y[0] = ry0;
         Y[1] = iy0;
      #else
         Y[0] += ry0;
         Y[1] += iy0;
      #endif
   }
}
